  MEMBER

  MAP
  END

  INCLUDE('JSRTIMER.INC'),ONCE

!========================================================================================
! JSR_TimerClass Methods
!========================================================================================

JSR_TimerClass.Init PROCEDURE(Window pWindow)
!
! Configures the object for use
!
  CODE
  IF Self.TimerQ &= NULL
    Self.LastID = 0
    Self.TimerQ       &= NEW JSR__TimerQType
    Self.WindowRef    &= pWindow
    Self.OriginalTimer = Self.WindowRef{Prop:Timer}
    Self._Add_My_Timers
  END
  RETURN

JSR_TimerClass.Kill PROCEDURE
!
! Removes all the timers and returns any allocated resources to Windows
!
  CODE
  IF Self.TimerQ &= NULL
    Self.UsageError('Kill')
  ELSE
    LOOP WHILE RECORDS(Self.TimerQ)
      GET(Self.TimerQ, 1)
      Self.KillOne
    END
    DISPOSE(Self.TimerQ)
  END
  RETURN

JSR_TimerClass.AddTimer PROCEDURE(long pDefaultInterval, string pTitle)
!
! Adds a Single Timer to the timer queue
!
  CODE
  IF Self.TimerQ &= NULL
    Self.UsageError('AddTimer(long,string)')
  ELSE
    Self.LastID += 1
    CLEAR(Self.TimerQ)
    Self.TimerQ.ID              = Self.LastID         ! Unique ID for this entry
    Self.TimerQ.DefaultInterval = pDefaultInterval    ! Default when it's turned on
    Self.TimerQ.Interval        = 0                   ! Not yet on
    Self.TimerQ.FireTime       = 0                   ! Time when next timer is to be fired
    Self.TimerQ.Title           = CLIP(pTitle)        ! Title for this timer
    ADD(Self.TimerQ)
  END
  RETURN Self.LastID

JSR_TimerClass.FindID PROCEDURE(long pID)
!
! Finds a specific timer ID
!
Lp      LONG
RetVal  LONG
  CODE
  RetVal = False                                    ! Assumes not found
  IF Self.TimerQ &= NULL                            ! Ensure proper method call order
    Self.UsageError('FindID(long)')
  ELSE
    IF Self.TimerQ.ID = pID                         ! If the ID we're looking for is the one currently in memory
      RetVal = True                                 ! then we don't need to loop thru the queue to look for it
    ELSE
      LOOP Lp = 1 TO RECORDS(Self.TimerQ)           ! Look thru all the timers
        GET(Self.TimerQ, Lp)
        IF Self.TimerQ.ID = pID                     ! To see if we can find our timer
          RetVal = True
          BREAK
        END
      END
    END
  END
  RETURN RetVal

JSR_TimerClass.Kill PROCEDURE(long pID)
!
! Remove a specific timer from the Timer queue
!
  CODE
  IF Self.TimerQ &= NULL                            ! Ensure proper method call order
    Self.UsageError('Kill(long)')
  ELSE
    IF Self.FindID(pID)                             ! Load the timer we're looking for into memeory
      Self.KillOne                                  ! And delete the queue entry currently active
    END
  END
  RETURN

JSR_TimerClass.KillOne PROCEDURE
  CODE
  IF Self.TimerQ &= NULL                            ! Ensure proper method call order
    Self.UsageError('KillOne')
  ELSE
    DELETE(Self.TimerQ)                             ! Removing a timer queue entry
  END
  RETURN

JSR_TimerClass.CheckTimers PROCEDURE
Lp                  LONG                            ! Loop variable
CurDate             LONG                            ! Current Date against which we are checking the timers
CurTime             LONG                            ! Current Time against which we are checking the timers
ClockDay            EQUATE(8640000)                 ! Number of 1/100th seconds in one day
OrigDisplayValue    CSTRING(256)

  CODE
  IF Self.TimerQ &= NULL                            ! Ensure proper method call order
    Self.UsageError('CheckTimers')
  ELSE
    !
    ! If there is anything in the display control (used to display what timer is active)
    ! save it off so we can put it back after were done looping thru the timer queue
    !
    IF Self.DisplayCtrl
      OrigDisplayValue = Self.DisplayCtrl{Prop:Text}
    END

    !
    !  Does anybody really know what time of day it is?
    !
    CurDate = TODAY()
    CurTime = CLOCK()

    LOOP Lp = 1 TO RECORDS(Self.TimerQ)
      GET(Self.TimerQ, Lp)
      !
      ! The title of the timer is displayed even when just checking to see if the timer should be fired
      !
      IF Self.DisplayCtrl
        Self.DisplayCtrl{Prop:Text} = Self.TimerQ.Title
        DISPLAY(Self.DisplayCtrl)
      END
      !
      ! If there is an interval (i.e., the timer is on) AND we have gone past the "FireTime" point
      !
      IF Self.TimerQ.Interval AND CurDate >= Self.TimerQ.FireDate AND CurTime >= Self.TimerQ.FireTime
        !
        ! Call the Timer Procedure
        !
        Self.ActiveTimer = Self.TimerQ.ID
        Self._Timer_Proc
        Self.ActiveTimer = 0
        !
        ! Since we have no control of what happens in the timer procedure, reload the correct
        ! TimerQ entry and set for the next timer
        !
        GET(Self.TimerQ, Lp)
        Self.TimerQ.FireDate = TODAY()
        Self.TimerQ.FireTime   = CLOCK() + Self.TimerQ.Interval
        IF Self.TimerQ.FireTime > ClockDay                          ! If the timer is beyond the end of the day
          Self.TimerQ.FireDate += 1                                 ! Set the fire date for the next day
          Self.TimerQ.FireTime -= ClockDay                          ! and subtrace one day frmo the time
        END
        PUT(Self.TimerQ)
      END
    END
    !
    ! Having finished looping thru all of the timer entries in the queue
    ! reset the display control back to it's original value
    !
    IF Self.DisplayCtrl
      Self.DisplayCtrl{Prop:Text} = OrigDisplayValue
      DISPLAY(Self.DisplayCtrl)
    END
  END
  RETURN

JSR_TimerClass.On PROCEDURE(long pID, <long pLock>)
!
! Turn a timer on
! Optionally, lock the timer so that the AllOff method can not turn if off
!
lLock   LONG
  CODE
  IF Self.TimerQ &= NULL                            ! Ensure proper method call order
    Self.UsageError('On(long,<long>)')
  ELSE
    lLock = False                                   ! Assume it won't be locked on
    IF NOT OMITTED(3)                               ! unless proven otherwise
      lLock = pLock                                 !
    END                                             !
    Self.SetTimer(pID, -1, lLock)                   ! -1 indicates to turn the timer on using it's default interval
  END
  RETURN

JSR_TimerClass.Off PROCEDURE(long pID)
!
! Turn a timer off
!
  CODE
  IF Self.TimerQ &= NULL                            ! Ensure proper method call order
    Self.UsageError('Off(long)')
  ELSE
    Self.SetTimer(pID, -2)                          ! -2 indicates to turn the timer off
  END
  RETURN

JSR_TimerClass.AllOff PROCEDURE
!
! Turn off all timers (except those that are locked on)
! (Timers that are locked on must be turned off individually be a call to the Off method)
!
Lp      LONG
RetVal  LONG
  CODE
  IF Self.TimerQ &= NULL                            ! Ensure proper method call order
    Self.UsageError('AllOff')
  ELSE
    !
    ! Loop thru all of the timers
    !
    LOOP Lp = 1 TO RECORDS(Self.TimerQ)
      GET(Self.TimerQ, Lp)
      IF NOT Self.TimerQ.Locked                     ! If they are Not locked on
        Self.TimerQ.Interval = 0                    ! <- This turns it off
        Self.TimerQ.FireDate = 0
        Self.TimerQ.FireTime = 0
        PUT(Self.TimerQ)
      END
    END
  END
  RETURN

JSR_TimerClass._Timer_Proc PROCEDURE
!
! This is the timer method that is internal to the object itself
! This method will call the Virtual method available to the specific instance of this object
!
  CODE
  IF Self.TimerQ &= NULL                            ! Ensure proper method call order
    Self.UsageError('_Timer_Proc')
  ELSE
    !
    ! Put code here that Must be done for Every timer event fired thru this method in your enterprise
    ! Note that if you do put anything here it must be redone with every update from Ripley Software
    !
    ! INCLUDE('MyTimer.INC','_Timer_Proc')

    !
    ! Now Call the Virtual Method
    !
    Self.Timer_Proc
  END
  RETURN

JSR_TimerClass.Timer_Proc PROCEDURE
!
! This is the virtual method (that must be overridden) that gets called whenever a timer event is being fired
!
  CODE
  MESSAGE('The JSR_TimerClass.Timer_Proc method should be overridden by the specific instance of this object.', |
          'Programming Error',icon:hand)
  RETURN

JSR_TimerClass._Add_My_Timers PROCEDURE
!
! This is the add timers method that is internal to the object itself
! This method will call the Virtual method available to the specific instance of this object
!
  CODE
  IF Self.TimerQ &= NULL                            ! Ensure proper method call order
    Self.UsageError('_Add_My_Timers')
  ELSE
    ! Put code here that Must be done for every timer object your enterprise
    ! Note that if you do put anything here it must be redone with every update from Ripley Software
    !
    ! INCLUDE('MyTimer.INC','_Add_My_Timers')

    !
    ! Now Call the virtual method
    Self.Add_My_Timers
  END

JSR_TimerClass.Add_My_Timers PROCEDURE
!
! This is the virtual method (that must be overridden) that is called by the Init method to add all the timers
!
  CODE
  MESSAGE('The JSR_TimerClass.Add_My_Timers method should be overridden by the specific instance of this object.', |
          'Programming Error',icon:hand)


JSR_TimerClass.SetTimer PROCEDURE(long pID, long pInterval, <long plock>)
!
! Set a specific timer
!   This sets the timer interval
!       -1 Indicates to turn the timer on using the default interval
!       -2 Indicates to turn the timer off
!       Any positive number specifies the defult interval but does NOT turn the timer on
!
!   And optionally locks the timer so that it can not be turned off by the AllOff method
!
Lp              LONG
SmallestTimer   LONG                    ! After all timers are evaluated, the smallest timer needed
                                        ! is used to set the Prop:Timer attribute on the window
NoTimer         EQUATE(07FFFFFFFh)      ! An Equate used to specify no timer
lLock           LONG                    ! Used when turning a timer on to lock its status to
                                        ! prevent it from being truned off by the AllOff Method
ClockDay        EQUATE(8640000)         ! Number of 1/100th seconds in a day
  CODE
  IF Self.TimerQ &= NULL                            ! Ensure proper method call order
    Self.UsageError('SetTimer(long,long,<long>)')
  ELSE
    lLock = False                                   ! Set the local Lock variable based
    IF NOT OMITTED(3)                               ! on what was (or wasn't) passed
      lLock = pLock
    END

    !
    ! SmallestTimer starts out at it's largest possible value, either NoTimer or whatever was on the
    ! window definition
    !
    IF NOT Self.OriginalTimer
      SmallestTimer = NoTimer
    ELSE
      SmallestTimer = Self.OriginalTimer
    END


    LOOP Lp = 1 TO RECORDS(Self.TimerQ)
      GET(Self.TimerQ, Lp)
      IF Self.TimerQ.ID = pID
        IF pInterval = -1
          !
          ! -1 = Turn the Timer On
          ! If it's not already On, set the timer to the default value from the last time it was set
          !
          IF NOT Self.TimerQ.Interval
            Self.TimerQ.Interval = Self.TimerQ.DefaultInterval
            Self.TimerQ.FireDate = TODAY()
            Self.TimerQ.FireTime = CLOCK() + Self.TimerQ.Interval
            IF Self.TimerQ.FireTime > ClockDay
              Self.TimerQ.FireDate += 1
              Self.TimerQ.FireTime -= ClockDay
            END
          END
          Self.TimerQ.Locked = lLock  ! Used to lock this timer on to prevent the AllOff method from turning it off
        ELSIF pInterval = -2
          !
          ! -2 = Turn the Timer Off
          !
          Self.TimerQ.Interval = 0
          Self.TimerQ.FireDate = 0
          Self.TimerQ.FireTime = 0
        ELSE
          !
          ! Set the interval and the default interval
          !
          Self.TimerQ.Interval = 0                        ! Not turned on yet
          Self.TimerQ.FireDate = 0
          Self.TimerQ.FireTime = 0
          Self.TimerQ.DefaultInterval = pInterval         ! Default interval
        END
        PUT(Self.TimerQ)
      END
      !
      ! The smallest interval needed will be used to set the windows prop:timer attribute
      !
      IF Self.TimerQ.Interval > 0 AND Self.TimerQ.Interval < SmallestTimer
        SmallestTimer = Self.TimerQ.Interval
      END
    END
    !
    ! If not timer was needed we will turn off the windows timer attribute
    !
    IF SmallestTimer = NoTimer
      SmallestTimer = 0
    END
    !
    ! Set the Prop:Timer attribute
    !
    IF Self.WindowRef{Prop:Timer} <> SmallestTimer        ! I'm not positive but if it's already the current value I don't
      Self.WindowRef{Prop:Timer} = SmallestTimer          ! want to set it because that may start the clock over again
                                                          ! for the Windows internal timer
    END
  END
  RETURN

JSR_TimerClass.UsageError PROCEDURE(string pMethod)
MethodName  CSTRING(80)
InitName    CSTRING(80)
  CODE
  IF Self.SelfName
    MethodName = Self.SelfName & '.' & CLIP(pMethod)
    InitName = Self.SelfName & '.Init'
  ELSE
    MethodName = 'JSR_TimerClass.' & CLIP(pMethod)
    InitName = 'JSR_TimerClass.Init'
  END
  MESSAGE(MethodName&' has been called before the '&InitName&' method.|' & |
         'Unpredictable results may follow, including GPF''s.','JSR_TimerClass Object Usage Error',icon:hand)
  RETURN

